@@ -85,7 +85,7 @@
 #define S_F_SEC_EPOCH		0x00000080
 #define S_F_HDR_ONLY		0x00000100
 #define S_F_FILE_LOCKED		0x00000200
-/* Unused			0x00000400*/
+#define S_F_SA_YYYYMMDD		0x00000400
 #define S_F_HORIZONTALLY	0x00000800
 #define S_F_COMMENT		0x00001000
 #define S_F_PERSIST_NAME	0x00002000
@@ -101,6 +101,7 @@
 #define PRINT_SEC_EPOCH(m)		(((m) & S_F_SEC_EPOCH)    == S_F_SEC_EPOCH)
 #define DISPLAY_HDR_ONLY(m)		(((m) & S_F_HDR_ONLY)     == S_F_HDR_ONLY)
 #define FILE_LOCKED(m)			(((m) & S_F_FILE_LOCKED)  == S_F_FILE_LOCKED)
+#define USE_SA_YYYYMMDD(m)		(((m) & S_F_SA_YYYYMMDD)  == S_F_SA_YYYYMMDD)
 #define DISPLAY_HORIZONTALLY(m)		(((m) & S_F_HORIZONTALLY) == S_F_HORIZONTALLY)
 #define DISPLAY_COMMENT(m)		(((m) & S_F_COMMENT)      == S_F_COMMENT)
 #define DISPLAY_PERSIST_NAME_S(m)	(((m) & S_F_PERSIST_NAME) == S_F_PERSIST_NAME)
@@ -242,11 +243,14 @@
  */
 #define AO_SELECTED		0x02
 /*
- * Indicate that, when registered again, activity counters will get back
- * the values they had when they were unregistered (eg. CPUs, which can
- * be disabled/enabled on the fly).
+ * When appending data to a file, the number of items (for every activity)
+ * is forced to that of the file (number of network interfaces, serial lines,
+ * etc.) Exceptions are volatile activities (like A_CPU) whose number of items
+ * is related to the number of CPUs: If current machine has a different number
+ * of CPU than that of the file (but is equal to sa_last_cpu_nr) then data
+ * will be appended with a number of items equal to that of the machine.
  */
-#define AO_REMANENT		0x04
+#define AO_VOLATILE		0x04
 /*
  * Indicate that the interval of time, given to f_print() function
  * displaying statistics, should be the interval of time in jiffies
@@ -268,7 +272,7 @@
 
 #define IS_COLLECTED(m)		(((m) & AO_COLLECTED)        == AO_COLLECTED)
 #define IS_SELECTED(m)		(((m) & AO_SELECTED)         == AO_SELECTED)
-#define IS_REMANENT(m)		(((m) & AO_REMANENT)         == AO_REMANENT)
+#define IS_VOLATILE(m)		(((m) & AO_VOLATILE)         == AO_VOLATILE)
 #define NEED_GLOBAL_ITV(m)	(((m) & AO_GLOBAL_ITV)       == AO_GLOBAL_ITV)
 #define CLOSE_MARKUP(m)		(((m) & AO_CLOSE_MARKUP)     == AO_CLOSE_MARKUP)
 #define HAS_MULTIPLE_OUTPUTS(m)	(((m) & AO_MULTIPLE_OUTPUTS) == AO_MULTIPLE_OUTPUTS)
@@ -298,7 +302,7 @@ struct act_bitmap {
 
 /*
  * Structure used to define an activity.
- * Note; This structure can be modified without changing the format of data files.
+ * Note: This structure can be modified without changing the format of data files.
  */
 struct activity {
 	/*
@@ -422,7 +426,7 @@ struct activity {
 	 */
 	unsigned int opt_flags;
 	/*
-	 * Buffers that will contain the statistics read. Its size is @nr * @size each.
+	 * Buffers that will contain the statistics read. Its size is @nr * @nr2 * @size each.
 	 * [0]: used by sadc.
 	 * [0] and [1]: current/previous statistics values (used by sar).
 	 * [2]: Used by sar to save first collected stats (used later to
@@ -451,7 +455,7 @@ struct activity {
  * 	|
  * 	|--                         --|
  * 	|                             |
- * 	| file_activity structure     | * sa_nr_act
+ * 	| file_activity structure     | * sa_act_nr
  * 	|                             |
  * 	|--                         --|
  * 	|                             |
@@ -464,8 +468,9 @@ struct activity {
  * 	|--                         --|
  *
  * (*)Note: If it's a special record, we may find a comment instead of
- * statistics (R_COMMENT record type) or even nothing at all (R_RESTART
- * record type).
+ * statistics (R_COMMENT record type) or, if it's a R_RESTART record type,
+ * <sa_nr_vol_act> structures (of type file_activity) for the volatile
+ * activities.
  ***************************************************************************
  */
 
@@ -480,7 +485,7 @@ struct activity {
  * Modified to indicate that the format of the file is
  * no longer compatible with that of previous sysstat versions.
  */
-#define FORMAT_MAGIC	0x2171
+#define FORMAT_MAGIC	0x2173
 
 /* Structure for file magic header data */
 struct file_magic {
@@ -499,6 +504,14 @@ struct file_magic {
 	unsigned char  sysstat_patchlevel;
 	unsigned char  sysstat_sublevel;
 	unsigned char  sysstat_extraversion;
+	/*
+	 * Size of file's header (size of file_header structure used by file).
+	 */
+	unsigned int header_size;
+	/*
+	 * Padding. Reserved for future use while avoiding a format change.
+	 */
+	unsigned char pad[64];
 };
 
 #define FILE_MAGIC_SIZE	(sizeof(struct file_magic))
@@ -511,9 +524,18 @@ struct file_header {
 	 */
 	unsigned long sa_ust_time	__attribute__ ((aligned (8)));
 	/*
-	 * Number of activities saved in the file
+	 * Number of CPU items (1 .. CPU_NR + 1) for the last sample in file.
 	 */
-	unsigned int sa_nr_act		__attribute__ ((aligned (8)));
+	unsigned int sa_last_cpu_nr	__attribute__ ((aligned (8)));
+	/*
+	 * Number of activities saved in file.
+	 */
+	unsigned int sa_act_nr;
+	/*
+	 * Number of volatile activities in file. This is the number of
+	 * file_activity structures saved after each restart mark in file.
+	 */
+	unsigned int sa_vol_act_nr;
 	/*
 	 * Current day, month and year.
 	 * No need to save DST (Daylight Saving Time) flag, since it is not taken
@@ -704,7 +726,7 @@ extern __nr_t
 	wrap_get_usb_nr(struct activity *);
 extern __nr_t
 	wrap_get_filesystem_nr(struct activity *);
-	
+
 /* Functions used to read activities statistics */
 extern __read_funct_t
 	wrap_read_stat_cpu(struct activity *);
@@ -786,6 +808,8 @@ extern void
 	allocate_bitmaps(struct activity * []);
 extern void
 	allocate_structures(struct activity * []);
+extern int
+	check_alt_sa_dir(char *, int, int);
 extern int
 	check_disk_reg(struct activity *, int, int, int);
 extern void
@@ -840,6 +864,11 @@ extern void
 	print_report_hdr(unsigned int, struct tm *, struct file_header *, int);
 extern void
 	read_file_stat_bunch(struct activity * [], int, int, int, struct file_activity *);
+extern __nr_t
+	read_vol_act_structures(int, struct activity * [], char *, struct file_magic *,
+			        unsigned int);
+extern int
+	reallocate_vol_act_structures(struct activity * [], unsigned int, unsigned int);
 extern int
 	sa_fread(int, void *, int, int);
 extern void
@@ -849,7 +878,7 @@ extern void
 extern void
 	set_bitmap(unsigned char [], unsigned char, unsigned int);
 extern void
-	set_default_file(struct tm *, char *, int);
+	set_default_file(char *, int, int);
 extern void
 	set_hdr_rectime(unsigned int, struct tm *, struct file_header *);
 
