@@ -107,8 +107,8 @@ void usage(char *progname)
 {
 	print_usage_title(stderr, progname);
 	fprintf(stderr, _("Options are:\n"
-			  "[ -A ] [ -B ] [ -b ] [ -C ] [ -d ] [ -F ] [ -H ] [ -h ] [ -p ] [ -q ] [ -R ]\n"
-			  "[ -r ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -V ] [ -v ] [ -W ] [ -w ] [ -y ]\n"
+			  "[ -A ] [ -B ] [ -b ] [ -C ] [ -D ] [ -d ] [ -F ] [ -H ] [ -h ] [ -p ] [ -q ]\n"
+			  "[ -R ] [ -r ] [ -S ] [ -t ] [ -u [ ALL ] ] [ -V ] [ -v ] [ -W ] [ -w ] [ -y ]\n"
 			  "[ -I { <int> [,...] | SUM | ALL | XALL } ] [ -P { <cpu> [,...] | ALL } ]\n"
 			  "[ -m { <keyword> [,...] | ALL } ] [ -n { <keyword> [,...] | ALL } ]\n"
 			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
@@ -202,7 +202,7 @@ void int_handler(int sig)
 void init_structures(void)
 {
 	int i;
-	
+
 	for (i = 0; i < 3; i++)
 		memset(&record_hdr[i], 0, RECORD_HEADER_SIZE);
 }
@@ -250,11 +250,11 @@ void print_read_error(void)
 void reverse_check_act(unsigned int act_nr)
 {
 	int i, j;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		
+
 		if (IS_SELECTED(act[i]->options)) {
-			
+
 			for (j = 0; j < act_nr; j++) {
 				if (id_seq[j] == act[i]->id)
 					break;
@@ -298,10 +298,10 @@ int sar_get_record_timestamp_struct(int curr)
 			 * The rectime structure has NOT been updated.
 			 */
 			return 1;
-		
+
 		rectime = *ltm;
 	}
-	
+
 	return 0;
 }
 
@@ -320,7 +320,7 @@ int check_line_hdr(void)
 	/* Get number of options entered on the command line */
 	if (get_activity_nr(act, AO_SELECTED, COUNT_OUTPUTS) > 1)
 		return TRUE;
-	
+
 	for (i = 0; i < NR_ACT; i++) {
 		if (IS_SELECTED(act[i]->options)) {
 			/* Special processing for activities using a bitmap */
@@ -387,7 +387,7 @@ void write_stats_avg(int curr, int read_from_file, unsigned int act_id)
 	int i;
 	unsigned long long itv, g_itv;
 	static __nr_t cpu_nr = -1;
-	
+
 	if (cpu_nr < 0)
 		cpu_nr = act[get_activity_position(act, A_CPU)]->nr;
 
@@ -402,15 +402,15 @@ void write_stats_avg(int curr, int read_from_file, unsigned int act_id)
 	strncpy(timestamp[curr], _("Average:"), TIMESTAMP_LEN);
 	timestamp[curr][TIMESTAMP_LEN - 1] = '\0';
 	strcpy(timestamp[!curr], timestamp[curr]);
-	
+
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		
+
 		if ((act_id != ALL_ACTIVITIES) && (act[i]->id != act_id))
 			continue;
-		
+
 		if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
 			/* Display current average activity statistics */
 			(*act[i]->f_print_avg)(act[i], 2, curr,
@@ -509,7 +509,7 @@ int write_stats(int curr, int read_from_file, long *cnt, int use_tm_start,
 	}
 
 	avg_count++;
-	
+
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
 
@@ -553,12 +553,12 @@ void write_stats_startup(int curr)
 			memset(act[i]->buf[!curr], 0, act[i]->msize * act[i]->nr * act[i]->nr2);
 		}
 	}
-	
+
 	flags |= S_F_SINCE_BOOT;
 	dis = TRUE;
-	
+
 	write_stats(curr, USE_SADC, &count, NO_TM_START, NO_TM_END, NO_RESET, ALL_ACTIVITIES);
-	
+
 	exit(0);
 }
 
@@ -608,15 +608,20 @@ int sa_read(void *buffer, int size)
  * @use_tm_end		Set to TRUE if option -e has been used.
  * @rtype		Record type to display.
  * @ifd			Input file descriptor.
+ * @file		Name of file being read.
+ * @file_magic		file_magic structure filled with file magic header
+ * 			data.
  *
  * RETURNS:
  * 1 if the record has been successfully displayed, and 0 otherwise.
  ***************************************************************************
  */
-int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int ifd)
+int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype,
+		      int ifd, char *file, struct file_magic *file_magic)
 {
 	char cur_time[26];
 	int dp = 1;
+	unsigned int new_cpu_nr;
 
 	if (set_record_timestamp_string(curr, cur_time, 26))
 		return 0;
@@ -628,8 +633,13 @@ int sar_print_special(int curr, int use_tm_start, int use_tm_end, int rtype, int
 	}
 
 	if (rtype == R_RESTART) {
+		/* Don't forget to read the volatile activities structures */
+		new_cpu_nr = read_vol_act_structures(ifd, act, file, file_magic,
+						     file_hdr.sa_vol_act_nr);
+
 		if (dp) {
-			printf("\n%-11s       LINUX RESTART\n", cur_time);
+			printf("\n%-11s       LINUX RESTART\t(%d CPU)\n",
+			       cur_time, new_cpu_nr > 1 ? new_cpu_nr - 1 : 1);
 			return 1;
 		}
 	}
@@ -665,9 +675,9 @@ void read_sadc_stat_bunch(int curr)
 	if (sa_read(&record_hdr[curr], RECORD_HEADER_SIZE)) {
 		print_read_error();
 	}
-	
+
 	for (i = 0; i < NR_ACT; i++) {
-		
+
 		if (!id_seq[i])
 			continue;
 		if ((p = get_activity_position(act, id_seq[i])) < 0) {
@@ -691,6 +701,8 @@ void read_sadc_stat_bunch(int curr)
  * @rows	Number of rows of screen.
  * @act_id	Activity to display.
  * @file_actlst	List of activities in file.
+ * @file	Name of file being read.
+ * @file_magic	file_magic structure filled with file magic header data.
  *
  * OUT:
  * @curr	Index in array for next sample statistics.
@@ -702,7 +714,8 @@ void read_sadc_stat_bunch(int curr)
  */
 void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf,
 			   int rows, unsigned int act_id, int *reset,
-			   struct file_activity *file_actlst)
+			   struct file_activity *file_actlst, char *file,
+			   struct file_magic *file_magic)
 {
 	int p;
 	unsigned long lines = 0;
@@ -745,7 +758,7 @@ void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf
 
 		if (!*eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
 			/* Read the extra fields since it's not a special record */
-			read_file_stat_bunch(act, *curr, ifd, file_hdr.sa_nr_act, file_actlst);
+			read_file_stat_bunch(act, *curr, ifd, file_hdr.sa_act_nr, file_actlst);
 		}
 
 		if ((lines >= rows) || !lines) {
@@ -760,7 +773,7 @@ void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf
 			if (rtype == R_COMMENT) {
 				/* Display comment */
 				next = sar_print_special(*curr, tm_start.use, tm_end.use,
-						     R_COMMENT, ifd);
+						     R_COMMENT, ifd, file, file_magic);
 				if (next) {
 					/* A line of comment was actually displayed */
 					lines++;
@@ -829,14 +842,19 @@ void read_header_data(void)
 		exit(3);
 	}
 
-	/* Read header data */
+	/*
+	 * Read header data.
+	 * No need to take into account file_magic.header_size. We are sure that
+	 * sadc and sar are from the same version (we have checked FORMAT_MAGIC
+	 * but also VERSION above) and thus the size of file_header is FILE_HEADER_SIZE.
+	 */
 	if (sa_read(&file_hdr, FILE_HEADER_SIZE)) {
 		print_read_error();
 	}
-	
+
 	/* Read activity list */
-	for (i = 0; i < file_hdr.sa_nr_act; i++) {
-		
+	for (i = 0; i < file_hdr.sa_act_nr; i++) {
+
 		if (sa_read(&file_act, FILE_ACTIVITY_SIZE)) {
 			print_read_error();
 		}
@@ -860,9 +878,9 @@ void read_header_data(void)
 	while (i < NR_ACT) {
 		id_seq[i++] = 0;
 	}
-	
+
 	/* Check that all selected activties are actually sent by sadc */
-	reverse_check_act(file_hdr.sa_nr_act);
+	reverse_check_act(file_hdr.sa_act_nr);
 }
 
 /*
@@ -910,14 +928,15 @@ void read_stats_from_file(char from_file[])
 
 			rtype = record_hdr[0].record_type;
 			if ((rtype == R_RESTART) || (rtype == R_COMMENT)) {
-				sar_print_special(0, tm_start.use, tm_end.use, rtype, ifd);
+				sar_print_special(0, tm_start.use, tm_end.use, rtype,
+						  ifd, from_file, &file_magic);
 			}
 			else {
 				/*
 				 * OK: Previous record was not a special one.
 				 * So read now the extra fields.
 				 */
-				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_nr_act,
+				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_act_nr,
 						     file_actlst);
 				if (sar_get_record_timestamp_struct(0))
 					/*
@@ -950,30 +969,32 @@ void read_stats_from_file(char from_file[])
 
 			if (!id_seq[i])
 				continue;
-			
+
 			if ((p = get_activity_position(act, id_seq[i])) < 0) {
 				/* Should never happen */
 				PANIC(1);
 			}
 			if (!IS_SELECTED(act[p]->options))
 				continue;
-			
+
 			if (!HAS_MULTIPLE_OUTPUTS(act[p]->options)) {
 				handle_curr_act_stats(ifd, fpos, &curr, &cnt, &eosaf, rows,
-						      act[p]->id, &reset, file_actlst);
+						      act[p]->id, &reset, file_actlst,
+						      from_file, &file_magic);
 			}
 			else {
 				unsigned int optf, msk;
-				
+
 				optf = act[p]->opt_flags;
-				
+
 				for (msk = 1; msk < 0x10; msk <<= 1) {
 					if (act[p]->opt_flags & msk) {
 						act[p]->opt_flags &= msk;
-						
+
 						handle_curr_act_stats(ifd, fpos, &curr, &cnt,
 								      &eosaf, rows, act[p]->id,
-								      &reset, file_actlst);
+								      &reset, file_actlst,
+								      from_file, &file_magic);
 						act[p]->opt_flags = optf;
 					}
 				}
@@ -987,12 +1008,13 @@ void read_stats_from_file(char from_file[])
 						 SOFT_SIZE);
 				rtype = record_hdr[curr].record_type;
 				if (!eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
-					read_file_stat_bunch(act, curr, ifd, file_hdr.sa_nr_act,
+					read_file_stat_bunch(act, curr, ifd, file_hdr.sa_act_nr,
 							     file_actlst);
 				}
 				else if (!eosaf && (rtype == R_COMMENT)) {
 					/* This was a COMMENT record: print it */
-					sar_print_special(curr, tm_start.use, tm_end.use, R_COMMENT, ifd);
+					sar_print_special(curr, tm_start.use, tm_end.use, R_COMMENT,
+							  ifd, from_file, &file_magic);
 				}
 			}
 			while (!eosaf && (rtype != R_RESTART));
@@ -1000,13 +1022,14 @@ void read_stats_from_file(char from_file[])
 
 		/* The last record we read was a RESTART one: Print it */
 		if (!eosaf && (record_hdr[curr].record_type == R_RESTART)) {
-			sar_print_special(curr, tm_start.use, tm_end.use, R_RESTART, ifd);
+			sar_print_special(curr, tm_start.use, tm_end.use, R_RESTART,
+					  ifd, from_file, &file_magic);
 		}
 	}
 	while (!eosaf);
 
 	close(ifd);
-	
+
 	free(file_actlst);
 }
 
@@ -1132,7 +1155,7 @@ int main(int argc, char **argv)
 #endif
 
 	tm_start.use = tm_end.use = FALSE;
-	
+
 	/* Allocate and init activity bitmaps */
 	allocate_bitmaps(act);
 
@@ -1153,6 +1176,12 @@ int main(int argc, char **argv)
 			}
 		}
 
+		else if (!strcmp(argv[opt], "-D")) {
+			/* Option to tell sar to write to saYYYYMMDD data files */
+			flags |= S_F_SA_YYYYMMDD;
+			opt++;
+		}
+
 		else if (!strcmp(argv[opt], "-P")) {
 			/* Parse -P option */
 			if (parse_sa_P_opt(argv, &opt, &flags, act)) {
@@ -1161,6 +1190,10 @@ int main(int argc, char **argv)
 		}
 
 		else if (!strcmp(argv[opt], "-o")) {
+			if (to_file[0]) {
+				/* Output file already specified */
+				usage(argv[0]);
+			}
 			/* Save stats to a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
@@ -1173,14 +1206,20 @@ int main(int argc, char **argv)
 		}
 
 		else if (!strcmp(argv[opt], "-f")) {
+			if (from_file[0] || day_offset) {
+				/* Input file already specified */
+				usage(argv[0]);
+			}
 			/* Read stats from a file */
 			if ((argv[++opt]) && strncmp(argv[opt], "-", 1) &&
 			    (strspn(argv[opt], DIGITS) != strlen(argv[opt]))) {
 				strncpy(from_file, argv[opt++], MAX_FILE_LEN);
 				from_file[MAX_FILE_LEN - 1] = '\0';
+				/* Check if this is an alternate directory for sa files */
+				check_alt_sa_dir(from_file, day_offset, -1);
 			}
 			else {
-				set_default_file(&rectime, from_file, day_offset);
+				set_default_file(from_file, day_offset, -1);
 			}
 		}
 
@@ -1237,11 +1276,15 @@ int main(int argc, char **argv)
 				usage(argv[0]);
 			}
 		}
-		
+
 		else if ((strlen(argv[opt]) > 1) &&
 			 (strlen(argv[opt]) < 4) &&
 			 !strncmp(argv[opt], "-", 1) &&
 			 (strspn(argv[opt] + 1, DIGITS) == (strlen(argv[opt]) - 1))) {
+			if (from_file[0] || day_offset) {
+				/* Input file already specified */
+				usage(argv[0]);
+			}
 			day_offset = atoi(argv[opt++] + 1);
 		}
 
@@ -1287,7 +1330,7 @@ int main(int argc, char **argv)
 	/* 'sar' is equivalent to 'sar -f' */
 	if ((argc == 1) ||
 	    ((interval < 0) && !from_file[0] && !to_file[0])) {
-		set_default_file(&rectime, from_file, day_offset);
+		set_default_file(from_file, day_offset, -1);
 	}
 
 	if (tm_start.use && tm_end.use && (tm_end.tm_hour < tm_start.tm_hour)) {
@@ -1312,7 +1355,7 @@ int main(int argc, char **argv)
 	if (!interval && (from_file[0] || to_file[0])) {
 		usage(argv[0]);
 	}
-	
+
 	/* Cannot enter a day shift with -o option */
 	if (to_file[0] && day_offset) {
 		usage(argv[0]);
@@ -1321,7 +1364,7 @@ int main(int argc, char **argv)
 	if (USE_PRETTY_OPTION(flags)) {
 		dm_major = get_devmap_major();
 	}
-	
+
 	if (!count) {
 		/*
 		 * count parameter not set: Display all the contents of the file
@@ -1332,7 +1375,7 @@ int main(int argc, char **argv)
 
 	/* Default is CPU activity... */
 	select_default_activity(act);
-	
+
 	/* Reading stats from file: */
 	if (from_file[0]) {
 		if (interval < 0) {
@@ -1341,7 +1384,7 @@ int main(int argc, char **argv)
 
 		/* Read stats from file */
 		read_stats_from_file(from_file);
-		
+
 		/* Free stuctures and activity bitmaps */
 		free_bitmaps(act);
 		free_structures(act);
@@ -1397,9 +1440,13 @@ int main(int argc, char **argv)
 
 		/* Flags to be passed to sadc */
 		salloc(args_idx++, "-z");
-		
+
 		/* Writing data to a file (option -o) */
 		if (to_file[0]) {
+			/* Set option -D if entered */
+			if (USE_SA_YYYYMMDD(flags)) {
+				salloc(args_idx++, "-D");
+			}
 			/* Collect all possible activities (option -S XALL for sadc) */
 			salloc(args_idx++, "-S");
 			salloc(args_idx++, K_XALL);
@@ -1412,7 +1459,7 @@ int main(int argc, char **argv)
 			 * to collect only activities that will be displayed.
 			 */
 			int act_id = 0;
-			
+
 			for (i = 0; i < NR_ACT; i++) {
 				if (IS_SELECTED(act[i]->options)) {
 					act_id |= act[i]->group;
